#include <t89c51ac2.h>   // SFR Beschreibungen
#include <stdio.h>       /* prototype declarations for I/O functions */
#include <stdlib.h>

/************************ Funktionsdeklarationen *********************/
void ser9611(void);    /*serielle Schnittstelle initialisieren */
void seriell_isr(void);
void extern_isr(void);
bit sende(unsigned char);
void sende_string(unsigned char*);
void clear_answer(void);
void rechnen(int);
void int_to_string(int,char*);
void init_extern0_interrupt(void);

/*********************** globale Variablen ***************************/
bit bSende=0, bEmpfang=0;			// Systemvariable fr serielle Schnittstelle
unsigned char uEmpfangsStelle;	// Systemvariable fr serielle Schnittstelle 
unsigned char xdata uEmpfangsdaten[256];			 // Feld fr 256 empfangenen Zeichen


/************************ Hauptprogramm ******************************/
void main(void)         
{
   long iZahl;

	ser9611();     // ser. Schnittstelle mit 9600Bd und fosz=11,0592
   init_extern0_interrupt();  // externer Interrupt wird eingestellt
	sende_string("\n\nDas Programm berechnet Primzahl.\n");
   	
	while(1){	 // Endlos Schleife erforderlich fr C Programmierung
      sende_string("Bis zur welchen Zahl sollen die Primzahlen berechnet werden? .. ");
      while(bEmpfang==0);
      bEmpfang=0; // Damit der Empfang der naechsten Daten angezeigt werden kann
      iZahl = atoi(uEmpfangsdaten); // Wandel String to int	
      clear_answer();
      rechnen(iZahl);
	} //Ende while
}



void ser9611(void)
// Datenbertragung 9600 Baud; Quarz = 11,0592 MHz; 
//1Start+8Daten+1Stopp; BA 1 und Timer 1
{
	SCON=0x40;	//1Start+8Daten+1Stopp; BA 1
	PCON |= 0x80;//	  //Baudraten verdopplung
	TR1=0; //Timer 1 stopp
	ET1=0; // Timer 1 Interrupt disabled
	TF1=0; // Timer TF1 geloescht
	TMOD &= 0x0F; //Timer 0 unbeeinflusst, 
	TMOD |= 0x20; // Timer 1 8 bit auto
	TH1=0xFA;	  // Reloadwert
	TL1=0xFA;
	REN=1;		  // Receive enable
	TR1 = 1;		  // Timer 1 start
	IEN0 |= 0x90;		// Interrupteinstellungen freigegeben
} 

///////////////////////////////////////////////////////////////////////
/******************* init_extern0_interrupt ***************************
Aufgabe: Stellt den externen Interrupt ein
Parameter:	
	keine
Rckgabe:	
	keinen
Ruft folgende Funktionen auf:
	keine				
Bentigt folgende globale Variablen:
	
Hardwareresourcen:
	Pin 3_2
///////////////////////////////////////////////////////////////////////
**********************************************************************/ 
void init_extern0_interrupt(void)
{
   IEN0 |= 0x81;  // externen Interrupt freischalten
   IT0 = 1;       // flankengesteuert
}

///////////////////////////////////////////////////////////////////////
/********************** Funktion clear_answer *************************
Aufgabe: Lscht den Empfangsdaten String
Parameter:	
	keine
Rckgabe:	
	keinen
Ruft folgende Funktionen auf:
	keine				
Bentigt folgende globale Variablen:
	uEmpfangsStelle, Empfangsdaten
Hardwareresourcen:
	ISR-UART
///////////////////////////////////////////////////////////////////////
**********************************************************************/ 
void clear_answer()
{
	uEmpfangsStelle=0;
	uEmpfangsdaten[uEmpfangsStelle]='\0';
}	

/**********************************************************************
************************* Interupt Service Routinen *******************
**********************************************************************/

seriell_isr() interrupt	4 
{
	if (RI != 0) {
   	if (SBUF != 0x0D) {
      	   uEmpfangsdaten[uEmpfangsStelle]=SBUF;
         	uEmpfangsStelle++;
     	} else {
				uEmpfangsdaten[uEmpfangsStelle]='\0';
				bEmpfang=1;
		}	 	  
		RI = 0;
	} // end if RI
	if (TI != 0) {
		bSende = 0;
		TI = 0;
	}
}



///////////////////////////////////////////////////////////////////////
/************************* Externer Interrupt *************************
Aufgabe: Sobald das Pin3_2 auf Low gelegt wird, wird ein externer 
Interrupt ausgelst. Die Abarbeitung des Programms des Mikrocontrollers
wird durch den Interrupt unterbrochen. Nun springt der Mikrocontroller 
in dieser ISR (Interrupt Service Routine).
Parameter:  keins
Rckgabe:   keins
Ruft folgende Funktionen auf:
            keine
Hardwareresourcen:
            externer Interrupt
///////////////////////////////////////////////////////////////////////
**********************************************************************/
extern_isr( ) interrupt 0
{
   P1=0x55;
}


///////////////////////////////////////////////////////////////////////
/************************** Funktion sende ****************************
Aufgabe: Sendet bergebenen Parameter an UART
Parameter:	Das zu versendene Byte
Rckgabe:	1 => Zeichen wird gesendet
			 	0 => Zeichen konnte noch nicht gesendet werden.
Ruft folgende Funktionen auf:
				keine
Bentigt folgende globale Variablen:
			   bSende
Hardwareresourcen:
				ISR-UART
///////////////////////////////////////////////////////////////////////
**********************************************************************/ 
bit sende(unsigned char uZeichen)
{
	if (bSende == 0) {
		SBUF = uZeichen;
		bSende = 1;
		return(1);
	}
	else
		return(0);
}

///////////////////////////////////////////////////////////////////////
/********************** Funktion sende_string *************************
Aufgabe: Sendet einen String ber UART
Parameter:	Zeiger auf den String
Rckgabe:	keine
Ruft folgende Funktionen auf:
				sende()							
Bentigt folgende globale Variablen:
				keine
Hardwareresourcen:
				keine
///////////////////////////////////////////////////////////////////////
**********************************************************************/
void sende_string(unsigned char *pcText)
{
	unsigned int data i;

	for (i=0; *(pcText+i)!='\0'; i++) {
		while(!sende(*(pcText+i)));
	}
}


/////////////////////////////////////////////////////////////////////////
/************************** Funktion rchnen(int) ************************
Aufgabe: Die Funktion rechnen berechnet alle Primzahlen bis zur bergebenden
Zahl und gibt diese ber die serielle Schnittstelle aus.
Parameter: Die Zahl bis zur der die Primzahlen berechnet werden soll
Rckgabe: kein

Ruft folgende Funktionen auf:
      int_to_string; wandelt eine Zahl in ein String
//////////////////////////////////////////////////////////////////////////
*************************************************************************/
void rechnen(int lZahl)
{
   int i,j;
   char sText[10];
   bit bPrimzahl;

   for(j=2; j<lZahl; j++) 
   {
      bPrimzahl=1;  
      for(i=2; i<j; i++)
      {
         if (j % i==0) 
         {  
            bPrimzahl=0;
         }
      }
      if(bPrimzahl == 1)
      {  
         int_to_string(i,sText);  //alternative: sprintf (sText, "%d",i); 
         sende_string(sText);
         sende_string("\n");
      }
   }
}


///////////////////////////////////////////////////////////////////////
/*********************** Funktion int_to_string ***********************
Aufgabe: Wandelt int Wert in ein String
Parameter:	
Rckgabe:	
	keins
Ruft folgende Funktionen auf:
	clear_answer(); sende_string();				
Bentigt folgende globale Variablen:
	bEmpfang, uEmpfangsdaten[],  
Hardwareresourcen:
	ISR-UART
///////////////////////////////////////////////////////////////////////
**********************************************************************/ 
void int_to_string(int iZahl, char *pString)
{
	char i, cTemp;

	if (iZahl >= 10000) i=4;
	else if (iZahl >= 1000) i=3;
	else if (iZahl >= 100) i=2;
	else if (iZahl >= 10) i=1;
	else i=0;
	*(pString+i+1) = '\0';
	while(i>=0) {
		cTemp = (iZahl % 10) + 0x30;
		*(pString+i) = cTemp;
		iZahl = iZahl / 10;
		i--;
	}
	
}
